#include <cassert>

#include <cstddef>
using std::size_t;

#include <iostream>
using std::cout;
using std::cerr;
using std::endl;

#include <fstream>
using std::ofstream;

#include <iomanip>
using std::setw;

#include <cmath>

#include <vector>
using std::vector;

#include <cmath>
using std::sin;
using std::cos;
using std::atan2;

#include "particle.hpp"
#include "ode_solver.h"

// acceleration model:

const double K = 1.0;

double HookesLaw(const particle &p)
{
	return -(K/p.mass)*p.x;
}

double pendulum(const particle &p)
{
	return -(K/p.mass)*sin(p.x);
}

double enclosedArea(const vector<particle> &p)
{
	double A = 0.0;
	for (size_t j = 0; j < p.size(); ++j)
	{
		const size_t k = (j+1)%p.size();
		const double R2_j = p[j].x*p[j].x + p[j].v*p[j].v;
		const double R2_k = p[k].x*p[k].x + p[k].v*p[k].v;
		const double theta_j = atan2(p[j].v,p[j].x);
		const double theta_k = atan2(p[k].v,p[k].x);
		A += (R2_k+R2_j)*sin(theta_k-theta_j);
	}
	return 0.25*A;
}

int main(int argc, char *argv[])
{
	assert(argc == 2);

	const double duration = 100.0; // run simulation for 100 s
	const unsigned int steps = 10000; 
	const double dt = duration / steps; // dt = 0.01;
	const unsigned int print_interval = 10;
	const size_t num_pendula = atoi(argv[1]); // read number of pendula from command line

	cerr << "Simulating " << num_pendula << " pendula with time step dt = " << dt << endl;
	
	vector<particle> p; // declare an empty (size 0) vector of particles
	for (size_t k = 0; k < num_pendula; ++k)
	{
		const double theta = 2.0*M_PI*k/num_pendula;
		double R = 3.0;// 3.0+2.0*cos(4.0*theta);
		double x = R*cos(theta);
		double v = R*sin(theta);
		p.push_back( particle(1.0,x,v) ); // add a particle to the end of the vector p
	}
	assert( p.size() == num_pendula );
	
	ofstream fout1("pendula.dat");
	ofstream fout2("area.dat");
	for (unsigned long int n = 0; n < steps; ++n)
	{
		if (n%print_interval == 0)
		{
			for (size_t k = 0; k < num_pendula; ++k)
				fout1 << p[k] << endl;
			fout1 << p[0] << endl;
			fout1 << endl << endl;
			fout2 << enclosedArea(p) << endl;
			
		}
		for (size_t k = 0; k < num_pendula; ++k)
		{
			//p[k].evolve(HookesLaw,Euler,dt);
			p[k].evolve(pendulum,Verlet,dt);
		}
	}
	fout1.close();
	fout2.close();

	return 0;
}