#include <cassert>

#include <iostream>
using std::cout;
using std::endl;

#include <iomanip>
using std::setw;

#include <limits>
using std::numeric_limits;

#include <cmath>
using std::cos;
using std::exp;
using std::log;

template <typename T>
T trapezoidIntegrator( T (&f)(T), T a, T b, unsigned int N)
{
	assert(N > 0);
	// N+1 = 2+n = 2,3,4,5,6,...
	
	const T width = b-a;
	const T h = width/N;
	T sum = 0.5*( f(a) + f(b) );
	for (unsigned int i = 1; i < N; ++i)
	{
		const T x_i = a + i*h;
		sum += f(x_i);
	}
	return sum*h;
}

template <typename T>
T SimpsonIntegrator( T (&f)(T), T a, T b, unsigned int N)
{
	assert(N > 1);
	assert(N%2 == 0);
	// N+1 = 3+2n = 3,5,7,9,11,...
	
	const T width = b-a;
	const T h = width/N;
	T sum = f(a) + 4*f(a+h) + f(b);
	for (unsigned int i = 2; i < N; i += 2)
	{
		const T x_i = a + i*h;
		sum += 2*f(x_i) + 4*f(x_i+h);
	}
	return sum*h/3.0;
}

template <typename T>
T BooleIntegrator( T (&f)(T), T a, T b, unsigned int N)
{
	assert(N > 3);
	assert(N%4 == 0);
	// N+1 = 5+4n = 5,9,13,17,21,...
	
	const T width = b-a;
	const T h = width/N;
	T sum = 7*f(a) + 32*f(a+h) + 12*f(a+2*h) + 32*f(a+3*h) + 7*f(b);
	for (unsigned int i = 4; i < N; i += 4)
	{
		const T x_i = a + i*h;
		sum += 14*f(x_i) + 32*f(x_i+h) + 12*f(x_i+2*h) + 32*f(x_i+3*h);
	}
	return 2.0*sum*h/45.0;
}

double myFunc(double x)
{
	const double y = cos(M_PI*x);
	return y*y*x*exp(x);
}

double myFunc2(double x)
{
	const double y = cos(M_PI*x);
	return y*y*x*exp(x)*log(x);
}

long double myFunc2(long double x)
{
	const long double y = cosl(M_PI*x);
	return y*y*x*expl(x)*logl(x);
}

double myFunc3(double x)
{
	return 1.0/(1.0+x*x);
}

long double myFunc3(long double x)
{
	return 1.0/(1.0+x*x);
}

const unsigned int pre_d = numeric_limits<double>::digits10;
const unsigned int pad_d = pre_d+8;

const unsigned int pre_ld = numeric_limits<long double>::digits10;
const unsigned int pad_ld = pre_d+8;

int main()
{
	const double eps = 1.0E-15;
	cout.precision(pre_d);
	for (unsigned int N = 10; N <= 5120; N *= 2)
	{
		const double step_size = 2.0/N;
		cout << setw(10) << N << setw(18) << step_size << setw(pad_d) << trapezoidIntegrator(myFunc,0.0,2.5,N) << endl;
	}
	
	cout << endl << endl;
	
	for (unsigned int N = 10; N <= 5120; N *= 2)
	{
		const double step_size = 2.0/N;
		cout << setw(10) << N << setw(18) << step_size << setw(pad_d) << SimpsonIntegrator(myFunc,0.0,2.5,N) << endl;
	}

	cout << endl << endl;
	
	for (unsigned int N = 20; N <= 5120; N *= 2)
	{
		const double step_size = 2.0/N;
		cout << setw(10) << N << setw(18) << step_size << setw(pad_d) << BooleIntegrator(myFunc,0.0,2.5,N) << endl;
	}

	cout << endl << endl;
	
	for (unsigned int N = 20; N <= 163840; N *= 2)
	{
		const double step_size = 2.0/N;
		const double left = ((1.0/N)/N)/N;
		cout << setw(10) << N << setw(18) << step_size << setw(pad_d) << BooleIntegrator(myFunc2,left,2.5,N) << endl;
	}
	
	cout << endl << endl;

	cout.precision(pre_ld);	
	for (unsigned int N = 20; N <= 163840; N *= 2)
	{
		const long double step_size = 2.0L/N;
		const long double left = ((1.0L/N)/N)/N;
		cout << setw(10) << N << setw(18) << step_size << setw(pad_ld) << BooleIntegrator(myFunc2,left,2.5L,N) << endl;
	}

	cout << endl << endl;
	
	cout.precision(pre_d);
	for (unsigned int N = 20; N <= 5120; N *= 2)
	{
		const double step_size = 2.0/N;
		double sum = BooleIntegrator(myFunc3,0.0,1.0,N);
		double x = 1.0;
		double w = 1.0;
		for (unsigned int M = 0; M < N; ++M)
		{
			const double y = w+x;
			sum += BooleIntegrator(myFunc3,x,y,N);
			x = y;
			w *= 1.1;
		}
		cout << setw(10) << N << setw(18) << step_size << setw(pad_d) << 2.0*sum << endl;			
	}

	cout << endl << endl;

	cout.precision(pre_ld);
	for (unsigned int N = 20; N <= 10240; N *= 2)
	{
		const long double step_size = 2.0L/N;
		long double sum = BooleIntegrator(myFunc3,0.0,1.0,N);
		long double x = 1.0;
		long double w = 1.0;
		for (unsigned int M = 0; M < N; ++M)
		{
			const long double y = w+x;
			sum += BooleIntegrator(myFunc3,x,y,N);
			x = y;
			w *= 1.1;
		}
		cout << setw(10) << N << setw(18) << step_size << setw(pad_d) << 2.0*sum << endl;			
	}
	
	return 0;
}
